/******************************************************************************
*
* LASER POSITION SENSOR
*
* Filename:		LPS.h
* Author:		  Joe Grand, Grand Idea Studio, Inc. [www.grandideastudio.com]
*
* Description:	Header File (Freescale MC9S08JM8)
*
* Distributed under a Creative Commons Attribution-Share Alike 3.0 Unported license:
* http://creativecommons.org/licenses/by-sa/3.0/
*
*******************************************************************************/

#ifndef __LPS__
#define __LPS__

#include <MC9S08JM8.h> /* include peripheral declarations */


/**************************************************************************
************************** Definitions ************************************
***************************************************************************/

#define LOW     0
#define HIGH    1

#define ON      1
#define OFF     0

#define _OUT    1
#define _IN     0
 
 // PWM
#define PWM_PERIOD      40000   // 400Hz @ 16MHz bus clock (divide-by-1 prescale)
#define PWM_DUTYCYCLE   20000   // 50% (default)


// timer 1
#define T1_SAMPLERATE   40000   // 100Hz @ 16MHz bus clock (divide-by-4 prescale)

    
// inputs
#define SW_EN     PTGD_PTGD2

   
// LEDs
#define LEDR      PTFD_PTFD4
#define LEDG      PTFD_PTFD5

#define LEDR_DD   PTFDD_PTFDD4
#define LEDG_DD   PTFDD_PTFDD5

#define LEDR_DS   PTFDS_PTFDS4
#define LEDG_DS   PTFDS_PTFDS5


// A/D channels
// Hamamatsu S5991-01 Position Sensing Device
// L = 10mm = 393.7mils
// L/2 = 5mm = 196.85mils
#define X1_INPUT    9
#define X2_INPUT    5
#define Y1_INPUT    8
#define Y2_INPUT    4

#define	ADCRANGE	    5000	  // a/d range in mV

#define FILTERCUTOFF	5 	    // hertz
#define COEF_A			  23871	  // A = (2/ts - 2PI*f) / (2/ts + 2PI*f) * 32768	
#define COEF_B			  4448	  // B = 2PI*f / (2/ts + 2PI*f) * 32768
#define DET_LENGTH	  1968		// length of detector/2 in tenths of mils for calc. X and Y values
#define	Pmw_COEF	    787 		// coeff for calc laser power (Fig. 4i in Circuit Cellar #152, March 2003)
#define LOWSUMLEVEL	  100			// sum low level in a/d counts
#define	SATLEVEL	    4050		// a/d channel saturation level in a/d counts


/**************************************************************************
************************** Macros *****************************************
***************************************************************************/


/**************************************************************************
************************** Structs ****************************************
***************************************************************************/

enum ERRORS {NOERROR, DETSAT, LOWSUM};		// errors
char const errorlevels[3][7] = { "      ","DETSAT","LOWSUM" };	// error messages

struct ad_data		// holds raw or filtered raw a/d data
{
	signed int x_1;
	signed int x_2;
	signed int y_1;
	signed int y_2;
	int error;
} rawdata = {0,0,0,0,NOERROR};		// holds the current filtered a/d data

struct ad_data v_offsets = {0,0,0,0,NOERROR};	      // holds the voltage offset data
struct ad_data last_ad_sample = {0,0,0,0,NOERROR};	// holds last ad sample for iir filter

struct calddata		// holds calibrated data
{
	signed int x;
	signed int y;
} displaydata;
	
struct calddata d_offsets = {0,0};	// holds the offsets for the x, y display data

struct filtercoefs
{
	signed long a;
	signed long b;
} coefs;	// holds the IIR filter coefficients


/***********************************************************************
 ************************** Function prototypes ************************
 ***********************************************************************/

// main.c
signed int iir_filter_data(signed int olddata, signed int newdata, signed int lastsample);
void compute_xy(void);
void display_xy(void);
void getvoltageoffset(void);

void HW_Init(void);
void delay_ms(unsigned int ms);
void delay(unsigned int count); 

#endif /* __LPS__ */
